@@ -20,10 +20,12 @@ from pytorch_lightning.metrics.functional.classification import (
     to_onehot,
     iou,
 )
+from pytorch_lightning.metrics.functional.nlp import bleu_score
 from pytorch_lightning.metrics.functional.regression import (
     mae,
     mse,
     psnr,
     rmse,
-    rmsle
+    rmsle,
+    ssim
 )
