@@ -1,3 +1,19 @@
+# Copyright The PyTorch Lightning team.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
+from typing import Sequence
+
 import torch
 
 from pytorch_lightning.metrics.functional.regression import (
@@ -5,7 +21,8 @@ from pytorch_lightning.metrics.functional.regression import (
     mse,
     psnr,
     rmse,
-    rmsle
+    rmsle,
+    ssim
 )
 from pytorch_lightning.metrics.metric import Metric
 
@@ -30,7 +47,7 @@ class MSE(Metric):
     ):
         """
         Args:
-            reduction: a method for reducing mse over labels (default: takes the mean)
+            reduction: a method to reduce metric score over labels (default: takes the mean)
                 Available reduction methods:
                 - elementwise_mean: takes the mean
                 - none: pass array
@@ -73,7 +90,7 @@ class RMSE(Metric):
     ):
         """
         Args:
-            reduction: a method for reducing mse over labels (default: takes the mean)
+            reduction: a method to reduce metric score over labels (default: takes the mean)
                 Available reduction methods:
                 - elementwise_mean: takes the mean
                 - none: pass array
@@ -98,7 +115,7 @@ class RMSE(Metric):
 
 class MAE(Metric):
     """
-    Computes the root mean absolute loss or L1-loss.
+    Computes the mean absolute loss or L1-loss.
 
     Example:
 
@@ -116,7 +133,7 @@ class MAE(Metric):
     ):
         """
         Args:
-            reduction: a method for reducing mse over labels (default: takes the mean)
+            reduction: a method to reduce metric score over labels (default: takes the mean)
                 Available reduction methods:
                 - elementwise_mean: takes the mean
                 - none: pass array
@@ -159,7 +176,7 @@ class RMSLE(Metric):
     ):
         """
         Args:
-            reduction: a method for reducing mse over labels (default: takes the mean)
+            reduction: a method to reduce metric score over labels (default: takes the mean)
                 Available reduction methods:
                 - elementwise_mean: takes the mean
                 - none: pass array
@@ -206,7 +223,7 @@ class PSNR(Metric):
         Args:
             data_range: the range of the data. If None, it is determined from the data (max - min)
             base: a base of a logarithm to use (default: 10)
-            reduction: method for reducing psnr (default: takes the mean)
+            reduction: a method to reduce metric score over labels (default: takes the mean)
                 Available reduction methods:
                 - elementwise_mean: takes the mean
                 - none: pass array
@@ -229,3 +246,62 @@ class PSNR(Metric):
             A Tensor with psnr score.
         """
         return psnr(pred, target, self.data_range, self.base, self.reduction)
+
+
+class SSIM(Metric):
+    """
+    Computes Structual Similarity Index Measure
+
+    Example:
+
+        >>> pred = torch.rand([16, 1, 16, 16])
+        >>> target = pred * 0.75
+        >>> metric = SSIM()
+        >>> metric(pred, target)
+        tensor(0.9219)
+
+    """
+
+    def __init__(
+            self,
+            kernel_size: Sequence[int] = (11, 11),
+            sigma: Sequence[float] = (1.5, 1.5),
+            reduction: str = "elementwise_mean",
+            data_range: float = None,
+            k1: float = 0.01,
+            k2: float = 0.03
+    ):
+        """
+        Args:
+            kernel_size: Size of the gaussian kernel (default: (11, 11))
+            sigma: Standard deviation of the gaussian kernel (default: (1.5, 1.5))
+            reduction: a method to reduce metric score over labels (default: takes the mean)
+                Available reduction methods:
+                - elementwise_mean: takes the mean
+                - none: pass away
+                - sum: add elements
+
+            data_range: Range of the image. If ``None``, it is determined from the image (max - min)
+            k1: Parameter of SSIM. Default: 0.01
+            k2: Parameter of SSIM. Default: 0.03
+        """
+        super().__init__(name="ssim")
+        self.kernel_size = kernel_size
+        self.sigma = sigma
+        self.reduction = reduction
+        self.data_range = data_range
+        self.k1 = k1
+        self.k2 = k2
+
+    def forward(self, pred: torch.Tensor, target: torch.Tensor) -> torch.Tensor:
+        """
+        Actual metric computation
+
+        Args:
+            pred: Estimated image
+            target: Ground truth image
+
+        Return:
+            A Tensor with SSIM score.
+        """
+        return ssim(pred, target, self.kernel_size, self.sigma, self.reduction, self.data_range, self.k1, self.k2)
