@@ -1,3 +1,17 @@
+# Copyright The PyTorch Lightning team.
+#
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
+#
+#     http://www.apache.org/licenses/LICENSE-2.0
+#
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
+
 """Mirroring deprecated API"""
 
 from abc import ABC
@@ -6,45 +20,14 @@ from typing import Union
 from pytorch_lightning.utilities import rank_zero_warn
 
 
-class TrainerDeprecatedAPITillVer0_9(ABC):
-    progress_bar_dict: ...
-    progress_bar_callback: ...
-
-    def __init__(self):
-        super().__init__()  # mixin calls super too
-
-    @property
-    def show_progress_bar(self):
-        """Back compatibility, will be removed in v0.9.0"""
-        rank_zero_warn("Attribute `show_progress_bar` is now set by `progress_bar_refresh_rate` since v0.7.2"
-                       " and this method will be removed in v0.9.0", DeprecationWarning)
-        return self.progress_bar_callback and self.progress_bar_callback.refresh_rate >= 1
-
-    @show_progress_bar.setter
-    def show_progress_bar(self, tf):
-        """Back compatibility, will be removed in v0.9.0"""
-        rank_zero_warn("Attribute `show_progress_bar` is now set by `progress_bar_refresh_rate` since v0.7.2"
-                       " and this method will be removed in v0.9.0", DeprecationWarning)
-
-    @property
-    def training_tqdm_dict(self):
-        """Back compatibility, will be removed in v0.9.0"""
-        rank_zero_warn("`training_tqdm_dict` was renamed to `progress_bar_dict` in v0.7.3"
-                       " and this method will be removed in v0.9.0", DeprecationWarning)
-        return self.progress_bar_dict
-
-    @property
-    def num_tpu_cores(self):
-        """Back compatibility, will be removed in v0.9.0"""
-        rank_zero_warn("Attribute `num_tpu_cores` is now set by `tpu_cores` since v0.7.6"
-                       " and this argument will be removed in v0.9.0", DeprecationWarning)
-
-
 class TrainerDeprecatedAPITillVer0_10(ABC):
     limit_val_batches: Union[int, float]
     limit_test_batches: Union[int, float]
     limit_train_batches: Union[int, float]
     overfit_batches: Union[int, float]
+    is_global_zero: bool
+    _weights_save_path: str
+    weights_save_path: str
 
     def __init__(self):
         super().__init__()  # mixin calls super too
@@ -118,3 +101,17 @@ class TrainerDeprecatedAPITillVer0_10(ABC):
         rank_zero_warn("Attribute `proc_rank` is now set by `global_rank` since v0.8.0"
                        " and this method will be removed in v0.10.0", DeprecationWarning)
         self.global_rank = rank
+
+    @property
+    def ckpt_path(self) -> str:
+        """Back compatibility, will be removed in v0.10.0"""
+        rank_zero_warn("Attribute `ckpt_path` is now set by `weights_save_path` since v0.9.0"
+                       " and this method will be removed in v0.10.0", DeprecationWarning)
+        return self.weights_save_path if self.is_global_zero else None
+
+    @ckpt_path.setter
+    def ckpt_path(self, path: str):
+        """Back compatibility, will be removed in v0.10.0"""
+        rank_zero_warn("Attribute `ckpt_path` is now set by `weights_save_path` since v0.9.0"
+                       " and this method will be removed in v0.10.0", DeprecationWarning)
+        self._weights_save_path = path
